'
' Main OS program
' runs in low memory
' loads programs at 0x10000 (64K)
'
CON
  _clkfreq = 160_000_000
  baud = 230_400
  BUFSIZ = 256
  MAXLINE = 128
  PBASE = $1_0000
  
OBJ
  ser: "spin/SmartSerial"
  fs: "fs9p.cc"
  
VAR
  BYTE cmdline[MAXLINE]
  BYTE program[MAXLINE]
  BYTE buf[BUFSIZ]
  long myfd[4]
  
PUB osmain | r
  clkset($010007f8, 160_000_000)
  ser.start(63, 62, 0, baud)
  ser.printf("OS demo\n")
  r := fs.fs_init(@sendrecv)
  if r < 0
    ser.printf("fs_init returned error %d\n", r)
    
  repeat
       ser.printf("Enter command> ")
       getcmd
       runcmd

PUB getcmd | c, i
  i := 0
  cmdline[0] := 0
  repeat
    c := ser.rx
    if (c == 8 or c == 127)
      if i > 0
        --i
        ser.tx(c)
    elseif c == 13 or c == 10
        ser.tx(13)
        ser.tx(10)
        quit
    else
        cmdline[i++] := c
        ser.tx(c)
  cmdline[i] := 0

PUB runcmd | s, p, r
  s := @cmdline
  p := @program
  repeat while byte[s] <> " " and byte[s] <> 0
    r := byte[s++]
    byte[p++] := r
  repeat while cmdline[s] == " "
    s++

  byte[p] := 0
  
  if program[0] == 0
    return
  byte[p++] := "."
  byte[p++] := "b"
  byte[p++] := "i"
  byte[p++] := "z"
  byte[p] := 0

  'ser.printf("Running [%s]\n", @program)
  
  r := fs.fs_open(@myfd, @program, 0)
  if r < 0
    ser.printf("Unable to find %s\n", @program)
    return

  ' read file into memory
  p := PBASE
  repeat
    r := fs.fs_read(@myfd, p, BUFSIZ)
    if r > 0
      p += r
  until r =< 0
  fs.fs_close(@myfd)
  
  ' now run the code at PBASE
  if p == PBASE
    ser.printf("read failed\n")
    return
  'ser.printf("load done\n")
  
  r := coginit(%0_1_0000, PBASE, 0)
  if r < 0
    ser.printf("Unable to allocate COG\n")
    return
  ' wait for COG to finish
  repeat while cogbusy(r)
  'ser.printf("done\n")
  
PUB cogbusy(id) : r
  asm
    cogid id wc
    wrc r
  endasm

'' routine for transmitting and receiving 9P protocol buffers

PUB sendrecv(startbuf, endbuf, maxlen) | len, buf, i, left
  len := endbuf - startbuf
  buf := startbuf
  long[startbuf] := len

  '' transmit magic sequence for loadp2
  ser.tx($FF)
  ser.tx($01)
  repeat while len > 0
    ser.tx(byte[buf++])
    len--

  ' now get response
  buf := startbuf
  byte[buf++] := ser.rx
  byte[buf++] := ser.rx
  byte[buf++] := ser.rx
  byte[buf++] := ser.rx
  len := long[startbuf]
  left := len - 4
  repeat while left > 0
    byte[buf++] := ser.rx
    --left
  return len

PUB dumphex(buf, len) | i
  ser.printf("hex dump:\n")
  repeat i from 0 to len-1
    ser.hex(byte[buf], 2)
    ser.tx(" ")
    buf++
  ser.tx(13)
  ser.tx(10)

